/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.gui;

import java.io.IOException;

import com.google.common.base.Strings;
import com.google.gson.JsonParseException;
import net.minecraft.client.gui.*;
import net.minecraft.client.resources.I18n;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.world.classic.WorldGenSettings;
import net.dries007.tfc.world.classic.WorldGenSettings.WorldGenSettingsBuilder;

@SideOnly(Side.CLIENT)
public class GuiCustomizeWorld extends GuiScreen implements GuiSlider.FormatHelper, GuiPageButtonList.GuiResponder
{
    private static final int ID_DONE = 0;
    private static final int ID_CANCEL = 1;
    private static final int ID_DEFAULTS = 2;

    private static final int ID_SPAWN_FUZZ = 100;
    private static final int ID_FLAT_BEDROCK = 101;

    private static final int ID_RAVINE_RARITY = 102;
    private static final int ID_RAVINE_HEIGHT = 103;
    private static final int ID_RAVINE_VARIABILITY = 104;

    private static final int ID_SURFACE_RAVINE_RARITY = 105;
    private static final int ID_SURFACE_RAVINE_HEIGHT = 106;
    private static final int ID_SURFACE_RAVINE_VARIABILITY = 107;

    private static final int ID_RIVER_RAVINE_RARITY = 108;

    private final GuiCreateWorld parent;

    // Final by all but modifier
    private String title;
    private GuiButton doneBtn;
    private GuiButton cancelBtn;
    private GuiButton defaultsBtn;

    private GuiPageButtonList list;

    private WorldGenSettingsBuilder settings;

    public GuiCustomizeWorld(GuiCreateWorld guiCreateWorld, String prevSettings)
    {
        parent = guiCreateWorld;
        settings = loadSettings(prevSettings);
    }

    @Override
    public void func_73863_a(int mouseX, int mouseY, float partialTicks)
    {
        func_146276_q_();
        func_73732_a(field_146289_q, title, field_146294_l / 2, 8, 16777215);
        this.list.func_148128_a(mouseX, mouseY, partialTicks);
        super.func_73863_a(mouseX, mouseY, partialTicks);
    }

    @Override
    public void func_175321_a(int id, boolean value)
    {
        switch (id)
        {
            case ID_FLAT_BEDROCK:
                settings.flatBedrock = value;
                break;
        }
        update();
    }

    @Override
    public void func_175320_a(int id, float value)
    {
        switch (id)
        {
            case ID_SPAWN_FUZZ:
                settings.spawnFuzz = Math.round(value);
                break;

            case ID_RAVINE_RARITY:
                settings.ravineRarity = Math.round(value);
                break;
            case ID_RAVINE_HEIGHT:
                settings.ravineHeight = Math.round(value);
                break;
            case ID_RAVINE_VARIABILITY:
                settings.ravineVariability = Math.round(value);
                break;

            case ID_SURFACE_RAVINE_RARITY:
                settings.surfaceRavineRarity = Math.round(value);
                break;
            case ID_SURFACE_RAVINE_HEIGHT:
                settings.surfaceRavineHeight = Math.round(value);
                break;
            case ID_SURFACE_RAVINE_VARIABILITY:
                settings.surfaceRavineVariability = Math.round(value);
                break;

            case ID_RIVER_RAVINE_RARITY:
                settings.riverRavineRarity = Math.round(value);
                break;
        }
        update();
    }

    @Override
    protected void func_73869_a(char typedChar, int keyCode)
    {
        list.func_178062_a(typedChar, keyCode);
    }

    @Override
    protected void func_146284_a(GuiButton button) throws IOException
    {
        if (!button.field_146124_l) return;

        switch (button.field_146127_k)
        {
            case ID_DONE:
                parent.field_146334_a = settings.build().toString();
                field_146297_k.func_147108_a(parent);
                break;

            case ID_CANCEL:
                field_146297_k.func_147108_a(parent);
                break;

            case ID_DEFAULTS:
                settings = loadSettings("");
                break;
        }
        super.func_146284_a(button);
        update();
    }

    @Override
    public void func_73866_w_()
    {
        super.func_73866_w_();

        title = I18n.func_135052_a("options.customizeTitle");

        doneBtn = func_189646_b(new GuiButton(ID_DONE, field_146294_l / 2 - 155, field_146295_m - 28, 150, 20, I18n.func_135052_a("gui.done")));
        cancelBtn = func_189646_b(new GuiButton(ID_CANCEL, field_146294_l / 2 + 5, field_146295_m - 28, 150, 20, I18n.func_135052_a("gui.cancel")));
        defaultsBtn = func_189646_b(new GuiButton(ID_DEFAULTS, 20, 5, 80, 20, I18n.func_135052_a("createWorld.customize.custom.defaults")));

        GuiPageButtonList.GuiListEntry[] page1 = new GuiPageButtonList.GuiListEntry[] {
                new GuiPageButtonList.GuiSlideEntry(ID_SPAWN_FUZZ, I18n.func_135052_a("createWorld.customize.custom.spawnfuzz"), true, this, 0, 2500, settings.spawnFuzz),
                new GuiPageButtonList.GuiButtonEntry(ID_FLAT_BEDROCK, I18n.func_135052_a("createWorld.customize.custom.flatbedrock"), true, settings.flatBedrock),

                new GuiPageButtonList.GuiSlideEntry(ID_RAVINE_RARITY, I18n.func_135052_a("createWorld.customize.custom.ravineRarity"), true, this, 0, 250, settings.ravineRarity),
                new GuiPageButtonList.GuiSlideEntry(ID_RAVINE_HEIGHT, I18n.func_135052_a("createWorld.customize.custom.ravineHeight"), true, this, 0, 50, settings.ravineHeight),
                new GuiPageButtonList.GuiSlideEntry(ID_RAVINE_VARIABILITY, I18n.func_135052_a("createWorld.customize.custom.ravineVariability"), true, this, 0, 100, settings.ravineVariability),

                new GuiPageButtonList.GuiSlideEntry(ID_SURFACE_RAVINE_RARITY, I18n.func_135052_a("createWorld.customize.custom.surfaceRavineRarity"), true, this, 0, 250, settings.surfaceRavineRarity),
                new GuiPageButtonList.GuiSlideEntry(ID_SURFACE_RAVINE_HEIGHT, I18n.func_135052_a("createWorld.customize.custom.surfaceRavineHeight"), true, this, 0, 250, settings.surfaceRavineHeight),
                new GuiPageButtonList.GuiSlideEntry(ID_SURFACE_RAVINE_VARIABILITY, I18n.func_135052_a("createWorld.customize.custom.surfaceRavineVariability"), true, this, 0, 100, settings.surfaceRavineVariability),

                new GuiPageButtonList.GuiSlideEntry(ID_RIVER_RAVINE_RARITY, I18n.func_135052_a("createWorld.customize.custom.riverRavineRarity"), true, this, 0, 500, settings.riverRavineRarity),
        };
        list = new GuiPageButtonList(field_146297_k, field_146294_l, field_146295_m, 32, field_146295_m - 32, 25, this, new GuiPageButtonList.GuiListEntry[][] {
                page1
        });

        update();
    }

    @Override
    protected void func_73864_a(int mouseX, int mouseY, int mouseButton) throws IOException
    {
        super.func_73864_a(mouseX, mouseY, mouseButton);
        list.func_148179_a(mouseX, mouseY, mouseButton);
    }

    @Override
    protected void func_146286_b(int mouseX, int mouseY, int state)
    {
        super.func_146286_b(mouseX, mouseY, state);
        list.func_148181_b(mouseX, mouseY, state);
    }

    @Override
    public void func_146274_d() throws IOException
    {
        super.func_146274_d();
        this.list.func_178039_p();
    }

    @Override
    public void func_175319_a(int id, String value)
    {
        update();
    }

    @Override
    public String func_175318_a(int id, String name, float value)
    {
        return name + ": " + this.getFormattedValue(id, value);
    }

    private void update()
    {
        defaultsBtn.field_146124_l = !settings.isDefault();
    }

    private WorldGenSettingsBuilder loadSettings(String str)
    {
        if (!Strings.isNullOrEmpty(str))
        {
            try
            {
                return WorldGenSettings.fromString(str);
            }
            catch (JsonParseException e)
            {
                TerraFirmaCraft.getLog().error("Error parsing s: {}", str);
                TerraFirmaCraft.getLog().catching(e);
            }
        }
        return new WorldGenSettingsBuilder();
    }

    private String getFormattedValue(int id, float value)
    {
        switch (id)
        {
            case ID_SPAWN_FUZZ:
            case ID_RAVINE_RARITY:
            case ID_RAVINE_HEIGHT:
            case ID_RAVINE_VARIABILITY:
            case ID_SURFACE_RAVINE_RARITY:
            case ID_SURFACE_RAVINE_HEIGHT:
            case ID_SURFACE_RAVINE_VARIABILITY:
            case ID_RIVER_RAVINE_RARITY:
                return String.valueOf(Math.round(value));
        }
        return String.valueOf(value);
    }
}
