/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.world.classic.chunkdata;

import javax.annotation.Nullable;

import net.minecraft.block.state.IBlockState;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTUtil;
import net.minecraft.util.math.BlockPos;

import net.dries007.tfc.objects.Ore;
import net.dries007.tfc.util.OreSpawnData;

public class ChunkDataOreSpawned
{
    @Nullable
    public final Ore ore;
    @Nullable
    public final IBlockState state;
    public final OreSpawnData.SpawnSize size;
    public final Ore.Grade grade;
    public final BlockPos pos;
    public final int count;

    public ChunkDataOreSpawned(@Nullable Ore ore, @Nullable IBlockState state, OreSpawnData.SpawnSize size, Ore.Grade grade, BlockPos pos, int count)
    {
        this.ore = ore;
        this.state = state;
        this.size = size;
        this.grade = grade;
        this.pos = pos;
        this.count = count;
        if (ore == null && state == null) throw new IllegalArgumentException("Both ore and state can't be null.");
    }

    public ChunkDataOreSpawned(NBTTagCompound tag)
    {
        ore = tag.func_74764_b("ore") ? Ore.valueOf(tag.func_74779_i("ore").toUpperCase()) : null;
        state = tag.func_74764_b("state") ? NBTUtil.func_190008_d(tag.func_74775_l("state")) : null;
        grade = Ore.Grade.valueOf(tag.func_74779_i("grade").toUpperCase());
        size = OreSpawnData.SpawnSize.valueOf(tag.func_74779_i("size").toUpperCase());
        pos = NBTUtil.func_186861_c(tag.func_74775_l("pos"));
        count = tag.func_74762_e("count");
    }

    public NBTTagCompound serialize()
    {
        NBTTagCompound root = new NBTTagCompound();
        if (ore != null) root.func_74778_a("ore", ore.name());
        if (state != null) root.func_74782_a("state", NBTUtil.func_190009_a(new NBTTagCompound(), state));
        root.func_74778_a("grade", grade.name());
        root.func_74778_a("size", size.name());
        root.func_74782_a("pos", NBTUtil.func_186859_a(pos));
        root.func_74768_a("count", count);
        return root;
    }

    @Override
    public String toString()
    {
        return "ChunkDataOreSpawned{" +
            "ore=" + ore +
            ", state=" + state +
            ", size=" + size +
            ", grade=" + grade +
            ", pos=" + pos +
            ", count=" + count +
            '}';
    }
}
